// ignore_for_file: no_leading_underscores_for_local_identifiers, avoid_renaming_method_parameters

import 'dart:async';

import 'package:aargul_core/aargul_core.dart';

class AargulLicence extends AargulModule {
  final FutureOr<bool> Function(String licenceCode) validate;
  final Widget Function(void Function([FutureOr<String>?]) complete)? builder;
  final Widget Function(BuildContext context)? successBuilder;
  final Duration successMessageDuration;
  final String successMessage;
  final String failsMessage;
  final String loadingMessage;
  AargulLicence({
    required this.validate,
    this.successBuilder,
    this.successMessageDuration = const Duration(seconds: 5),
    this.successMessage = "Votre licence a été activée...",
    this.failsMessage = "Cette licence est invalide",
    this.loadingMessage = "Check lience...",
    this.builder,
  }) : super();

  @override
  void initModule(aargulContext) {
    var state = aargulContext.find<AargulModuleState>();
    Future.microtask(() async {
      // await state.remove("licence",scope: "licence");
      String licenceCode = "";
      if (!await state.has("licence", scope: "licence")) {
        Completer<String> _licenceCode = Completer();
        var _controller = TextEditingController(text: '');
        aargulContext.lock(builder?.call(_licenceCode.complete) ??
            Container(
              padding: const EdgeInsets.all(20),
              color: Colors.errorSecondaryColor,
              child: Column(
                mainAxisAlignment: MainAxisAlignment.center,
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisSize: MainAxisSize.max,
                children: [
                  const Text("Entrez la Licence"),
                  TextBox(
                    autofocus: true,
                    textAlign: TextAlign.center,
                    controller: _controller,
                    placeholder: 'Copiez le code de la licence',
                    onSubmitted: (v) {
                      if (!_licenceCode.isCompleted) {
                        _licenceCode.complete(_controller.text);
                      }
                    },
                    onEditingComplete: () {
                      if (!_licenceCode.isCompleted) {
                        _licenceCode.complete(_controller.text);
                      }
                    },
                    maxLines: 5,
                    minLines: 5,
                    style: const TextStyle(
                      fontSize: 24.0,
                      letterSpacing: 8.0,
                      color: Color(0xFF5178BE),
                      fontStyle: FontStyle.italic,
                    ),
                  ),
                  Button(
                      onPressed: () {
                        if (!_licenceCode.isCompleted) {
                          _licenceCode.complete(_controller.text);
                        }
                      },
                      child: const Text("Valider"))
                ],
              ),
            ));
        licenceCode = await _licenceCode.future;
      } else {
        licenceCode = await state.get("licence", scope: "licence");
      }

      aargulContext.release(); // show loader
      try {
        aargulContext.loadingMessage(loadingMessage);
        if (await validate.call(licenceCode)) {
          await state.set("licence", scope: "licence", licenceCode);
          aargulContext.lock(
              successBuilder != null
                  ? Builder(
                      builder: (context) => successBuilder!(context),
                    )
                  : Center(
                      child: Column(
                        mainAxisAlignment: MainAxisAlignment.center,
                        crossAxisAlignment: CrossAxisAlignment.center,
                        mainAxisSize: MainAxisSize.max,
                        children: [
                          const Icon(FluentIcons.accept, size: 24),
                          Text(successMessage),
                        ],
                      ),
                    ),
              successMessageDuration);
        } else {
          throw failsMessage;
        }
        Timer(const Duration(seconds: 1), () {
          aargulContext.next();
        });
      } catch (e) {
        aargulContext.next(e);
      }
    });
  }
}
